@@ -26,13 +26,13 @@ import logging
 import multiprocessing
 import queue
 import threading
+import six
 
 from .. import rpc, utils
 from ..ddap_protocol import AppDROPStates
 from ..drop import AppDROP, BarrierAppDROP
 from ..exceptions import InvalidDropException
 
-
 logger = logging.getLogger(__name__)
 
 _read_cb_type = ctypes.CFUNCTYPE(
@@ -93,6 +93,12 @@ class CDlgApp(ctypes.Structure):
         ("data", ctypes.c_void_p),
     ]
 
+    def pack_python(self):
+        out = {}
+        for key, val in self._fields_:
+            out[key] = repr(getattr(self, key))
+        return out
+
 
 def _to_c_input(i):
     """
@@ -347,6 +353,13 @@ class DynlibStreamApp(DynlibAppBase, AppDROP):
         super(DynlibStreamApp, self).addStreamingInput(streamingInputDrop, back)
         self._c_app.n_streaming_inputs += 1
 
+    def generate_recompute_data(self):
+        out = {"status": self.status}
+        data = self._c_app.pack_python()
+        if data is not None:
+            out.update(data)
+        return out
+
 
 ##
 # @brief DynlibApp
@@ -380,6 +393,14 @@ class DynlibApp(DynlibAppBase, BarrierAppDROP):
         self._ensure_c_outputs_are_set()
         run(self.lib, self._c_app, input_closers)
 
+    def generate_recompute_data(self):
+        out = {"status": self.status}
+        if self._c_app is None:
+            return out
+        else:
+            out.update(self._c_app.pack_python())
+            return out
+
 
 class FinishSubprocess(Exception):
     pass
@@ -438,6 +459,25 @@ def get_from_subprocess(proc, q):
     raise RuntimeError("Subprocess died unexpectedly")
 
 
+##
+# @brief DynlibProcApp
+# @details An application component run from a dynamic library in a different process
+# @par EAGLE_START
+# @param category DynlibProcApp
+# @param tag template
+# @param[in] cparam/libpath Library Path//String/readwrite/False//False/
+#     \~English The location of the shared object/DLL that implements this application
+# @param[in] cparam/execution_time Execution Time/5/Float/readonly/False//False/
+#     \~English Estimated execution time
+# @param[in] cparam/num_cpus No. of CPUs/1/Integer/readonly/False//False/
+#     \~English Number of cores used
+# @param[in] cparam/group_start Group start/False/Boolean/readwrite/False//False/
+#     \~English Is this node the start of a group?
+# @param[in] cparam/input_error_threshold "Input error rate (%)"/0/Integer/readwrite/False//False/
+#     \~English the allowed failure rate of the inputs (in percent), before this component goes to ERROR state and is not executed
+# @param[in] cparam/n_tries Number of tries/1/Integer/readwrite/False//False/
+#     \~English Specifies the number of times the 'run' method will be executed before finally giving up
+# @par EAGLE_END
 class DynlibProcApp(BarrierAppDROP):
     """Loads a dynamic library in a different process and runs it"""
 
