@@ -30,12 +30,12 @@ from . import constants
 from .client import NodeManagerClient
 from .constants import ISLAND_DEFAULT_REST_PORT, NODE_DEFAULT_REST_PORT
 from .drop_manager import DROPManager
-from .. import remote, graph_loader
+from .. import graph_loader
+from ..common.reproducibility.reproducibility import init_pg_repro_data
 from ..ddap_protocol import DROPRel
 from ..exceptions import InvalidGraphException, DaliugeException, SubManagerException
 from ..utils import portIsOpen
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -45,8 +45,15 @@ def uid_for_drop(dropSpec):
     return dropSpec["oid"]
 
 
-def sanitize_relations(interDMRelations, graph):
+def sanitize_link(link):
+    """
+    Links can now be dictionaries, but we only need
+    the key.
+    """
+    return list(link.keys())[0] if isinstance(link, dict) else link
 
+
+def sanitize_relations(interDMRelations, graph):
     # TODO: Big change required to remove this hack here
     #
     # Values in the interDMRelations array use OIDs to identify drops.
@@ -79,8 +86,11 @@ def sanitize_relations(interDMRelations, graph):
     # know about the OIDs.
     newDMRelations = []
     for rel in interDMRelations:
-        lhs = uid_for_drop(graph[rel.lhs])
-        rhs = uid_for_drop(graph[rel.rhs])
+        lhs = rel.lhs
+        lhs = sanitize_link(rel.lhs)
+        lhs = uid_for_drop(graph[lhs])
+        rhs = sanitize_link(rel.rhs)
+        rhs = uid_for_drop(graph[rhs])
         new_rel = DROPRel(lhs, rel.rel, rhs)
         newDMRelations.append(new_rel)
     interDMRelations[:] = newDMRelations
@@ -339,6 +349,15 @@ class CompositeManager(DROPManager):
         # attribute set
         logger.info(f"Separating graph using partition attribute {self._partitionAttr}")
         perPartition = collections.defaultdict(list)
+        if "rmode" in graphSpec[-1]:
+            init_pg_repro_data(graphSpec)
+            self._graph["reprodata"] = graphSpec.pop()
+            logger.debug(
+                "Composite manager found reprodata in dropspecList, rmode=%s",
+                self._graph["reprodata"]["rmode"],
+            )
+        if graphSpec[-1] == {}:
+            graphSpec.pop()
         for dropSpec in graphSpec:
             if self._partitionAttr not in dropSpec:
                 msg = "Drop %s doesn't specify a %s attribute" % (
@@ -360,7 +379,6 @@ class CompositeManager(DROPManager):
 
             # Add the drop specs to our graph
             self._graph[uid_for_drop(dropSpec)] = dropSpec
-
         # At each partition the relationships between DROPs should be local at the
         # moment of submitting the graph; thus we record the inter-partition
         # relationships separately and remove them from the original graph spec
@@ -391,6 +409,9 @@ class CompositeManager(DROPManager):
         # Create the individual graphs on each DM now that they are correctly
         # separated.
         logger.info("Adding individual graphSpec of session %s to each DM", sessionId)
+        for partition in perPartition:
+            if self._graph.get("reprodata") is not None:
+                perPartition[partition].append(self._graph["reprodata"])
         self.replicate(
             sessionId,
             self._addGraphSpec,
