@@ -68,6 +68,13 @@ class DROPManager(object):
         Returns the status of the session `sessionId`.
         """
 
+    @abc.abstractmethod
+    def getSessionReproStatus(self, sessionId):
+        """
+        Returns the reproducibility status of the session `sessionId`.
+        Not guaranteed to be identical to the usual SessionStatus.
+        """
+
     @abc.abstractmethod
     def addGraphSpec(self, sessionId, graphSpec):
         """
@@ -89,6 +96,12 @@ class DROPManager(object):
         `sessionId`.
         """
 
+    @abc.abstractmethod
+    def getGraphReproData(self, sessionId):
+        """
+        Returns the graph-wide reproducibility data for session `sessionId`
+        """
+
     @abc.abstractmethod
     def deploySession(self, sessionId, completedDrops=[]):
         """
