@@ -25,16 +25,22 @@ Module containing the logic of a session -- a given graph execution
 
 import collections
 import inspect
+import json
 import logging
+import os
 import threading
 import time
 import socket
 
+from dlg.common.reproducibility.reproducibility import init_runtime_repro_data
+from dlg.utils import createDirIfMissing
+
 from . import constants
 from .. import droputils
 from .. import graph_loader
 from .. import rpc
 from .. import utils
+from ..common.reproducibility.constants import ReproducibilityFlags, ALL_RMODES
 from ..ddap_protocol import DROPLinkType, DROPRel, DROPStates
 from ..drop import (
     AbstractDROP,
@@ -51,7 +57,6 @@ from ..exceptions import (
     DaliugeException,
 )
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -83,6 +88,25 @@ class LeavesCompletionListener(object):
             self._session.finish()
 
 
+class ReproFinishedListener(object):
+    def __init__(self, graph, session):
+        self._session = session
+        self._nexpected = len(graph)
+        self._completed = 0
+
+    def handleEvent(self, evt):
+        self._completed += 1
+        self._session.append_reprodata(evt.oid, evt.reprodata)
+        logger.debug(
+            "%d/%d drops filed reproducibility", self._completed, self._nexpected
+        )
+        if self._completed == self._nexpected:
+            logger.debug("Building Reproducibility BlockDAG")
+            init_runtime_repro_data(self._session._graph, self._session._graphreprodata)
+            self._session.reprostatus = True
+            self._session.write_reprodata()
+
+
 class EndListener(object):
     """
     Listener for an EndDROP that will end the session when complete
@@ -134,6 +158,8 @@ class Session(object):
         self._error_status_listener = None
         self._nm = nm
         self._dropsubs = {}
+        self._graphreprodata = None
+        self._reprofinished = False
 
         class SessionFilter(logging.Filter):
             def __init__(self, sessionId):
@@ -184,6 +210,27 @@ class Session(object):
     def drops(self):
         return self._drops
 
+    @property
+    def reprodata(self):
+        return self._graphreprodata
+
+    @property
+    def reprostatus(self):
+        return self._reprofinished
+
+    @reprostatus.setter
+    def reprostatus(self, status):
+        with self._statusLock:  # TODO: Consider creating another lock
+            self._reprofinished = status
+
+    def write_reprodata(self):
+        parts = [utils.getDlgLogsDir(), self._sessionId]
+        the_dir = os.path.abspath(os.path.normpath(os.path.join(*parts)))
+        createDirIfMissing(the_dir)
+        the_path = os.path.join(the_dir, "reprodata.out")
+        with open(the_path, "w+", encoding="utf-8") as file:
+            json.dump([self._graph, self._graphreprodata], file, indent=4)
+
     @track_current_session
     def addGraphSpec(self, graphSpec):
         """
@@ -193,7 +240,10 @@ class Session(object):
         DROP. Each DROP specification is checked to see it contains
         all the necessary details to construct a proper DROP. If one
         DROP specification is found to be inconsistent the whole operation
-        fill wail.
+        will fail.
+
+        This operation also 'slices off' a dictionary containing graph-wide
+        reproducibility information. This is stored as a class variable for later use.
 
         Adding graph specs to the session is only allowed while the session is
         in the PRISTINE or BUILDING status; otherwise an exception will be
@@ -214,8 +264,7 @@ class Session(object):
         self.status = SessionStates.BUILDING
 
         # This will check the consistency of each dropSpec
-        graphSpecDict = graph_loader.loadDropSpecs(graphSpec)
-
+        graphSpecDict, self._graphreprodata = graph_loader.loadDropSpecs(graphSpec)
         # Check for duplicates
         duplicates = set(graphSpecDict) & set(self._graph)
         if duplicates:
@@ -299,6 +348,9 @@ class Session(object):
         )
         logger.info("%d drops successfully created", len(self._graph))
 
+        #  Add listeners for reproducibility information
+        repro_listener = ReproFinishedListener(self._graph, self)
+
         for drop, _ in droputils.breadFirstTraverse(self._roots):
 
             # Register them
@@ -313,6 +365,8 @@ class Session(object):
             # Register them with the error handler
             for l in event_listeners:
                 drop.subscribe(l)
+            #  Register each drop for reproducibility listening
+            drop.subscribe(repro_listener, "reproducibility")
 
         logger.info("Stored all drops, proceeding with further customization")
 
@@ -391,7 +445,7 @@ class Session(object):
         Called when an event has been fired by a remote drop.
         The event is then delivered to the interested drops of this session.
         """
-        if not evt.uid in self._dropsubs:
+        if evt.uid not in self._dropsubs:
             logger.debug("No subscription found for drop %s", evt.uid)
             return
         for tgt in self._dropsubs[evt.uid]:
@@ -456,6 +510,29 @@ class Session(object):
 
                 self._proxyinfo.append((host, rpc_port, local_uid, mname, remote_uid))
 
+    def append_reprodata(self, oid, reprodata):
+        if oid in self._graph:
+            if self._graph[oid].get("reprodata") is None:
+                return
+            if self._graph[oid]["reprodata"]["rmode"] == str(
+                ReproducibilityFlags.ALL.value
+            ):
+                drop_reprodata = reprodata.get("data", {})
+                drop_hashes = reprodata.get("merkleroot", {})
+                for rmode in ALL_RMODES:
+                    self._graph[oid]["reprodata"][rmode.name][
+                        "rg_data"
+                    ] = drop_reprodata[rmode.name]
+                    self._graph[oid]["reprodata"][rmode.name]["rg_data"][
+                        "merkleroot"
+                    ] = drop_hashes.get(rmode.name, b"")
+
+            else:
+                self._graph[oid]["reprodata"]["rg_data"] = reprodata.get("data", {})
+                self._graph[oid]["reprodata"]["rg_data"]["merkleroot"] = reprodata.get(
+                    "merkleroot", b""
+                )
+
     @track_current_session
     def finish(self):
         self.status = SessionStates.FINISHED
