@@ -36,7 +36,6 @@ import zerorpc
 
 from . import utils
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -239,10 +238,8 @@ class ZeroRPCServer(RPCServerBase):
         # Use context provided by subclass
         self._zrpcserver = zerorpc.Server(self, context=self._context)
         # zmq needs an address, not a hostname
-        endpoint = "tcp://%s:%d" % (
-            utils.zmq_safe(host),
-            port,
-        )
+        endpoint = "tcp://%s:%d" % (utils.zmq_safe(host), port)
+        logger.debug("Trying to bind ZeroRPC to %s", endpoint)
         self._zrpcserver.bind(endpoint)
         logger.info("Listening for RPC requests via ZeroRPC on %s", endpoint)
         server_started.set()
